@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2018 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2019 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,8 +19,8 @@
 
 #define _ND_PLUGIN_VER  0x20180813
 
-#define StartDetectionThreads() kill(getpid(), SIGUSR1)
-#define StopDetectionThreads()  kill(getpid(), SIGUSR2)
+#define ndStartDetectionThreads() kill(getpid(), SIGUSR1)
+#define ndStopDetectionThreads()  kill(getpid(), SIGUSR2)
 
 #define ndPluginInit(class_name) \
     extern "C" { \
@@ -139,6 +139,7 @@ protected:
     void *so_handle;
     ndPlugin *plugin;
 };
+
 #endif // _ND_INTERNAL
 
 #endif // _ND_PLUGIN_H
