@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2018 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2019 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,10 +26,18 @@ public:
         : runtime_error(what_arg) { }
 };
 
+class ndThreadSystemException : public ndSystemException
+{
+public:
+    explicit ndThreadSystemException(
+        const string &where_arg, const string &what_arg, int why_arg) throw()
+        : ndSystemException(where_arg, what_arg, why_arg) { }
+};
+
 class ndThread
 {
 public:
-    ndThread(const string &tag, long cpu = -1);
+    ndThread(const string &tag, long cpu = -1, bool ipc = false);
     virtual ~ndThread();
 
     string GetTag(void) { return tag; }
@@ -49,15 +57,25 @@ public:
     void Lock(void);
     void Unlock(void);
 
+    void SendIPC(uint32_t id);
+    uint32_t RecvIPC(void);
+
 protected:
     string tag;
     pthread_t id;
     pthread_attr_t attr;
     long cpu;
-    atomic_bool terminate;
-    atomic_bool terminated;
+#ifdef HAVE_ATOMIC
+    atomic<bool> terminate;
+    atomic<bool> terminated;
+#else
+    bool terminate;
+    bool terminated;
+#endif
     pthread_mutex_t lock;
 
+    int fd_ipc[2];
+
     int Join(void);
 };
 
