@@ -1,4 +1,3 @@
-
 /*
  * ndpi_protocol_ids.h
  *
@@ -22,7 +21,6 @@
  *
  */
 
-
 #ifndef __NDPI_API_H__
 
 #endif
@@ -49,7 +47,7 @@ typedef enum {
     NDPI_PROTOCOL_BGP                   = 13,
     NDPI_PROTOCOL_SNMP                  = 14,
     NDPI_PROTOCOL_XDMCP                 = 15,
-  NDPI_PROTOCOL_SMBV1                 = 16, /* SMB version 1 */
+    NDPI_PROTOCOL_SMBV1                 = 16, /* SMB version 1 */
     NDPI_PROTOCOL_SYSLOG                = 17,
     NDPI_PROTOCOL_DHCP                  = 18,
     NDPI_PROTOCOL_POSTGRES              = 19,
@@ -71,19 +69,18 @@ typedef enum {
     NDPI_PROTOCOL_GNUTELLA              = 35,
     NDPI_PROTOCOL_EDONKEY               = 36, /* Tomasz Bujlow <tomasz@skatnet.dk> */
     NDPI_PROTOCOL_BITTORRENT            = 37,
-    NDPI_PROTOCOL_SKYPE_CALL_OUT        = 38,
-    NDPI_PROTOCOL_MUSICALLY             = 39,
+    NDPI_PROTOCOL_SKYPE_CALL            = 38, /* Skype call and videocalls */
+    NDPI_PROTOCOL_SIGNAL                = 39,
     NDPI_PROTOCOL_MEMCACHED             = 40, /* Memcached - Darryl Sokoloski <darryl@egloo.ca> */
     NDPI_PROTOCOL_SMBV23                = 41, /* SMB version 2/3 */
     NDPI_PROTOCOL_MINING                = 42, /* Bitcoin, Ethereum, ZCash, Monero */
     NDPI_PROTOCOL_NEST_LOG_SINK         = 43, /* Nest Log Sink (Nest Protect) - Darryl Sokoloski <darryl@egloo.ca> */
-    NDPI_PROTOCOL_FREE_44               = 44, /* Free */
-    NDPI_PROTOCOL_FREE_45               = 45, /* Free */
-    NDPI_PROTOCOL_FREE_46               = 46, /* Free */
-
+    NDPI_PROTOCOL_MODBUS                = 44, /* Modbus */
+    NDPI_PROTOCOL_WHATSAPP_VIDEO        = 45,
+    NDPI_PROTOCOL_DATASAVER             = 46, /* Protocols used to save data on Internet communications */
     NDPI_PROTOCOL_XBOX                  = 47,
     NDPI_PROTOCOL_QQ                    = 48,
-    NDPI_PROTOCOL_SKYPE_CALL_IN         = 49,
+    NDPI_PROTOCOL_TIKTOK                = 49,
     NDPI_PROTOCOL_RTSP                  = 50,
     NDPI_PROTOCOL_MAIL_IMAPS            = 51,
     NDPI_PROTOCOL_ICECAST               = 52,
@@ -230,7 +227,7 @@ typedef enum {
     NDPI_PROTOCOL_KAKAOTALK             = 193, /* KakaoTalk Chat (no voice call) */
     NDPI_PROTOCOL_KAKAOTALK_VOICE       = 194, /* KakaoTalk Voice */
     NDPI_PROTOCOL_TWITCH                = 195, /* Edoardo Dominici <edoaramis@gmail.com> */
-    NDPI_PROTOCOL_FREE_196              = 196, /* Free */
+    NDPI_PROTOCOL_HTTPS                 = 196,
     NDPI_PROTOCOL_WECHAT                = 197,
     NDPI_PROTOCOL_MPEGTS                = 198,
     NDPI_PROTOCOL_SNAPCHAT              = 199,
@@ -239,8 +236,8 @@ typedef enum {
     NDPI_PROTOCOL_IFLIX                 = 202, /* www.vizuamatix.com R&D team & M.Mallawaarachchie <manoj_ws@yahoo.com> */
     NDPI_PROTOCOL_GITHUB                = 203,
     NDPI_PROTOCOL_BJNP                  = 204,
-    NDPI_PROTOCOL_FREE_205              = 205, /* Free */
-    NDPI_PROTOCOL_VIDTO                 = 206,
+    NDPI_PROTOCOL_DNS_OVER_HTTPS        = 205,
+    NDPI_PROTOCOL_FREE_206              = 206, /* Free */
     NDPI_PROTOCOL_SMPP                  = 207, /* Damir Franusic <df@release14.org> */
     NDPI_PROTOCOL_DNSCRYPT              = 208,
     NDPI_PROTOCOL_TINC                  = 209, /* William Guglielmo <william@deselmo.com> */
@@ -270,8 +267,8 @@ typedef enum {
     NDPI_PROTOCOL_LINKEDIN              = 233, /* Paulo Angelo <pa@pauloangelo.com> */
     NDPI_PROTOCOL_SOUNDCLOUD            = 234,
     NDPI_PROTOCOL_CSGO                  = 235, /* Counter-Strike Global Offensive, Dota = 2 */
-    NDPI_PROTOCOL_LISP	              = 236,
-    NDPI_PROTOCOL_DIAMETER	      = 237,
+    NDPI_PROTOCOL_LISP                  = 236,
+    NDPI_PROTOCOL_DIAMETER              = 237,
     NDPI_PROTOCOL_APPLE_PUSH            = 238,
     NDPI_PROTOCOL_GOOGLE_SERVICES       = 239,
     NDPI_PROTOCOL_AMAZON_VIDEO          = 240,
