@@ -331,19 +331,19 @@ struct hash_ip4p {
 
 struct hash_ip4p_table {
     size_t                  size;
-    int			  ipv6;
+    int                     ipv6;
     spinlock_t              lock;
     atomic_t                count;
     struct hash_ip4p        tbl;
 };
 
 struct bt_announce {              // 192 bytes
-    u_int32_t		hash[5];
-    u_int32_t		ip[4];
-    u_int32_t		time;
-    u_int16_t		port;
-    u_int8_t		name_len,
-                    name[192 - 4*10 - 2 - 1];     // 149 bytes
+    u_int32_t               hash[5];
+    u_int32_t               ip[4];
+    u_int32_t               time;
+    u_int16_t               port;
+    u_int8_t                name_len,
+                            name[192 - 4*10 - 2 - 1];     // 149 bytes
 };
 
 /* NDPI_PROTOCOL_TINC */
@@ -356,17 +356,22 @@ PACK_ON struct tinc_cache_entry {
 } PACK_OFF;
 
 typedef enum {
-    HTTP_METHOD_UNKNOWN = 0,
-    HTTP_METHOD_OPTIONS,
-    HTTP_METHOD_GET,
-    HTTP_METHOD_HEAD,
-    HTTP_METHOD_POST,
-    HTTP_METHOD_PUT,
-    HTTP_METHOD_DELETE,
-    HTTP_METHOD_TRACE,
-    HTTP_METHOD_CONNECT
+    NDPI_HTTP_METHOD_UNKNOWN = 0,
+    NDPI_HTTP_METHOD_OPTIONS,
+    NDPI_HTTP_METHOD_GET,
+    NDPI_HTTP_METHOD_HEAD,
+    NDPI_HTTP_METHOD_PATCH,
+    NDPI_HTTP_METHOD_POST,
+    NDPI_HTTP_METHOD_PUT,
+    NDPI_HTTP_METHOD_DELETE,
+    NDPI_HTTP_METHOD_TRACE,
+    NDPI_HTTP_METHOD_CONNECT
 } ndpi_http_method;
 
+struct ndpi_lru_cache {
+    u_int32_t num_entries, *entries;
+};
+
 struct ndpi_id_struct {
     /* detected_protocol_bitmask:
      * access this bitmask to find out whether an id has used skype or not
@@ -513,7 +518,7 @@ struct ndpi_flow_tcp_struct {
     u_int32_t http_wait_for_retransmission:1;
 
     /* NDPI_PROTOCOL_GNUTELLA */
-    u_int32_t gnutella_stage:2;		       // 0 - 2
+    u_int32_t gnutella_stage:2; // 0 - 2
 
     /* NDPI_CONTENT_MMS */
     u_int32_t mms_stage:2;
@@ -530,13 +535,16 @@ struct ndpi_flow_tcp_struct {
     u_int32_t ssh_stage:3;
 
     /* NDPI_PROTOCOL_VNC */
-    u_int32_t vnc_stage:2;			// 0 - 3
+    u_int32_t vnc_stage:2; // 0 - 3
 
     /* NDPI_PROTOCOL_TELNET */
-    u_int32_t telnet_stage:2;			// 0 - 2
+    u_int32_t telnet_stage:2; // 0 - 2
 
     /* NDPI_PROTOCOL_SSL */
-    u_int8_t ssl_seen_client_cert:1, ssl_seen_server_cert:1, ssl_stage:2; // 0 - 5
+    u_int8_t ssl_seen_client_cert:1,
+             ssl_seen_server_cert:1,
+             ssl_seen_certificate:1,
+             ssl_stage:2; // 0 - 5
 
     /* NDPI_PROTOCOL_POSTGRES */
     u_int32_t postgres_stage:3;
@@ -602,69 +610,69 @@ struct ndpi_flow_tcp_struct {
 #ifndef WIN32
 __attribute__ ((__packed__))
 #endif
-;
+    ;
 
-/* ************************************************** */
+    /* ************************************************** */
 
-struct ndpi_flow_udp_struct {
-    /* NDPI_PROTOCOL_BATTLEFIELD */
-    u_int32_t battlefield_msg_id;
+    struct ndpi_flow_udp_struct {
+        /* NDPI_PROTOCOL_BATTLEFIELD */
+        u_int32_t battlefield_msg_id;
 
-    /* NDPI_PROTOCOL_SNMP */
-    u_int32_t snmp_msg_id;
+        /* NDPI_PROTOCOL_SNMP */
+        u_int32_t snmp_msg_id;
 
-    /* NDPI_PROTOCOL_BATTLEFIELD */
-    u_int32_t battlefield_stage:3;
+        /* NDPI_PROTOCOL_BATTLEFIELD */
+        u_int32_t battlefield_stage:3;
 
-    /* NDPI_PROTOCOL_SNMP */
-    u_int32_t snmp_stage:2;
+        /* NDPI_PROTOCOL_SNMP */
+        u_int32_t snmp_stage:2;
 
-    /* NDPI_PROTOCOL_PPSTREAM */
-    u_int32_t ppstream_stage:3;		  // 0 - 7
+        /* NDPI_PROTOCOL_PPSTREAM */
+        u_int32_t ppstream_stage:3; // 0 - 7
 
-    /* NDPI_PROTOCOL_HALFLIFE2 */
-    u_int32_t halflife2_stage:2;		  // 0 - 2
+        /* NDPI_PROTOCOL_HALFLIFE2 */
+        u_int32_t halflife2_stage:2; // 0 - 2
 
-    /* NDPI_PROTOCOL_TFTP */
-    u_int32_t tftp_stage:1;
+        /* NDPI_PROTOCOL_TFTP */
+        u_int32_t tftp_stage:1;
 
-    /* NDPI_PROTOCOL_AIMINI */
-    u_int32_t aimini_stage:5;
+        /* NDPI_PROTOCOL_AIMINI */
+        u_int32_t aimini_stage:5;
 
-    /* NDPI_PROTOCOL_XBOX */
-    u_int32_t xbox_stage:1;
+        /* NDPI_PROTOCOL_XBOX */
+        u_int32_t xbox_stage:1;
 
-    /* NDPI_PROTOCOL_WINDOWS_UPDATE */
-    u_int32_t wsus_stage:1;
+        /* NDPI_PROTOCOL_WINDOWS_UPDATE */
+        u_int32_t wsus_stage:1;
 
-    /* NDPI_PROTOCOL_SKYPE */
-    u_int8_t skype_packet_id;
+        /* NDPI_PROTOCOL_SKYPE */
+        u_int8_t skype_packet_id;
 
-    /* NDPI_PROTOCOL_TEAMVIEWER */
-    u_int8_t teamviewer_stage;
+        /* NDPI_PROTOCOL_TEAMVIEWER */
+        u_int8_t teamviewer_stage;
 
-    /* NDPI_PROTOCOL_EAQ */
-    u_int8_t eaq_pkt_id;
-    u_int32_t eaq_sequence;
+        /* NDPI_PROTOCOL_EAQ */
+        u_int8_t eaq_pkt_id;
+        u_int32_t eaq_sequence;
 
-    /* NDPI_PROTOCOL_RX */
-    u_int32_t rx_conn_epoch;
-    u_int32_t rx_conn_id;
+        /* NDPI_PROTOCOL_RX */
+        u_int32_t rx_conn_epoch;
+        u_int32_t rx_conn_id;
 
-    /* NDPI_PROTOCOL_MEMCACHED */
-    u_int8_t memcached_matches;
-}
+        /* NDPI_PROTOCOL_MEMCACHED */
+        u_int8_t memcached_matches;
+    }
 #ifndef WIN32
 __attribute__ ((__packed__))
 #endif
-;
+    ;
 
-/* ************************************************** */
+    /* ************************************************** */
 
-struct ndpi_int_one_line_struct {
-    const u_int8_t *ptr;
-    u_int16_t len;
-};
+    struct ndpi_int_one_line_struct {
+        const u_int8_t *ptr;
+        u_int16_t len;
+    };
 
 struct ndpi_packet_struct {
     const struct ndpi_iphdr *iph;
@@ -673,7 +681,7 @@ struct ndpi_packet_struct {
 #endif
     const struct ndpi_tcphdr *tcp;
     const struct ndpi_udphdr *udp;
-    const u_int8_t *generic_l4_ptr;	/* is set only for non tcp-udp traffic */
+    const u_int8_t *generic_l4_ptr; /* is set only for non tcp-udp traffic */
     const u_int8_t *payload;
 
     u_int32_t tick_timestamp;
@@ -685,7 +693,7 @@ struct ndpi_packet_struct {
     __attribute__ ((__packed__))
 #endif
 
-    struct ndpi_int_one_line_struct line[NDPI_MAX_PARSE_LINES_PER_PACKET];
+        struct ndpi_int_one_line_struct line[NDPI_MAX_PARSE_LINES_PER_PACKET];
 
     /* HTTP headers */
     struct ndpi_int_one_line_struct host_line;
@@ -744,20 +752,36 @@ typedef struct {
 } ndpi_port_range;
 
 typedef enum {
-    NDPI_BREED_NULL = 0
+    NDPI_PROTOCOL_UNRATED = 0
 } ndpi_protocol_breed_t;
 
 /* Abstract categories to group the protocols. */
 typedef enum {
-  NDPI_CATEGORY_NULL = 0
+    NDPI_PROTOCOL_CATEGORY_UNSPECIFIED = 0,
+
+    /*
+       IMPORTANT
+
+       Please keep in sync with
+
+       static const char* categories[] = { ..}
+
+       in ndpi_main.c
+       */
+
+    NDPI_PROTOCOL_NUM_CATEGORIES /*
+                                    NOTE: Keep this as last member
+                                    Unused as value but useful to getting the number of elements
+                                    in this datastructure
+                                    */
 } ndpi_protocol_category_t;
 
 typedef enum {
     ndpi_pref_http_dont_dissect_response = 0,
-    ndpi_pref_dns_dissect_response,
+    ndpi_pref_dns_dont_dissect_response,
     ndpi_pref_direction_detect_disable,
     ndpi_pref_disable_metadata_export,
-    ndpi_pref_enable_category_substring_match,
+    ndpi_pref_enable_category_substring_match
 } ndpi_detection_preference;
 
 /* ntop extensions */
@@ -888,12 +912,15 @@ struct ndpi_detection_module_struct {
     struct bt_announce *bt_ann;
     int    bt_ann_len;
 
+    /* NDPI_PROTOCOL_OOKLA */
+    struct ndpi_lru_cache *ookla_cache;
+
     /* NDPI_PROTOCOL_TINC */
     struct cache *tinc_cache;
 
     ndpi_proto_defaults_t proto_defaults[NDPI_MAX_SUPPORTED_PROTOCOLS+NDPI_MAX_NUM_CUSTOM_PROTOCOLS];
 
-    u_int8_t http_dont_dissect_response:1, dns_dissect_response:1,
+    u_int8_t http_dont_dissect_response:1, dns_dont_dissect_response:1,
              direction_detect_disable:1, /* disable internal detection of packet direction */
              disable_metadata_export:1, /* No metadata is exported */
              enable_category_substring_match:1 /* Default is perfect match */
@@ -904,14 +931,20 @@ struct ndpi_detection_module_struct {
 
 #endif /* NDPI_LIB_COMPILATION */
 
+typedef enum {
+    ndpi_cipher_safe = NDPI_CIPHER_SAFE,
+    ndpi_cipher_weak = NDPI_CIPHER_WEAK,
+    ndpi_cipher_insecure = NDPI_CIPHER_INSECURE
+} ndpi_cipher_weakness;
+
 struct ndpi_flow_struct {
     u_int16_t detected_protocol;
 #ifndef WIN32
     __attribute__ ((__packed__))
 #endif
 
-    /* init parameter, internal used to set up timestamp,... */
-    u_int16_t guessed_proto_id, guessed_host_proto_id;
+        /* init parameter, internal used to set up timestamp,... */
+        u_int16_t guessed_proto_id, guessed_host_proto_id;
     u_int8_t proto_id_already_guessed:1, host_already_guessed:1, init_finished:1, setup_packet_direction:1, packet_direction:1, check_extra_packets:1;
 
     /*
@@ -922,6 +955,7 @@ struct ndpi_flow_struct {
 
     u_int8_t max_extra_packets_to_check;
     u_int8_t num_extra_packets_checked;
+    u_int8_t num_processed_pkts; /* <= WARNING it can wrap but we do expect people to giveup earlier */
 
     int (*extra_packets_func) (struct ndpi_detection_module_struct *, struct ndpi_flow_struct *flow);
 
@@ -954,7 +988,7 @@ struct ndpi_flow_struct {
         char *url, *content_type;
         u_int8_t num_request_headers, num_response_headers;
         u_int8_t request_version; /* 0=1.0 and 1=1.1. Create an enum for this? */
-        u_char response_status_code[5]; /* 200, 404, etc. */
+        u_int16_t response_status_code; /* 200, 404, etc. */
     } http;
 
     union {
@@ -962,6 +996,7 @@ struct ndpi_flow_struct {
         struct {
             u_int8_t num_queries, num_answers, reply_code;
             u_int16_t query_type, query_class, rsp_type;
+            ndpi_ip_addr_t rsp_addr; /* The first address in a DNS response packet */
         } dns;
 
         struct {
@@ -971,8 +1006,11 @@ struct ndpi_flow_struct {
 
         struct {
             struct {
-                u_int16_t version, cipher_suite;
-                char client_certificate[48], server_certificate[48];
+                u_int16_t ssl_version;
+                char client_certificate[64], server_certificate[64], server_organization[64];
+                char ja3_client[33], ja3_server[33];
+                u_int16_t version, server_cipher;
+                ndpi_cipher_weakness server_unsafe_cipher;
             } ssl;
 
             struct {
@@ -1023,73 +1061,66 @@ struct ndpi_flow_struct {
     /* NDPI_PROTOCOL_REDIS */
     u_int8_t redis_s2d_first_char, redis_d2s_first_char;
 
-    u_int16_t packet_counter;		      // can be 0 - 65000
+    u_int16_t packet_counter; // can be 0 - 65000
     u_int16_t packet_direction_counter[2];
     u_int16_t byte_counter[2];
     /* NDPI_PROTOCOL_BITTORRENT */
-    u_int8_t bittorrent_stage;		      // can be 0 - 255
+    u_int8_t bittorrent_stage; // can be 0 - 255
 
     /* NDPI_PROTOCOL_DIRECTCONNECT */
-    u_int32_t directconnect_stage:2;	      // 0 - 1
+    u_int8_t directconnect_stage:2; // 0 - 1
 
     /* NDPI_PROTOCOL_YAHOO */
-    u_int32_t sip_yahoo_voice:1;
+    u_int8_t sip_yahoo_voice:1;
 
     /* NDPI_PROTOCOL_HTTP */
-    u_int32_t http_detected:1;
+    u_int8_t http_detected:1;
+    u_int16_t http_upper_protocol, http_lower_protocol;
 
     /* NDPI_PROTOCOL_RTSP */
-    u_int32_t rtsprdt_stage:2;
-    u_int32_t rtsp_control_flow:1;
+    u_int8_t rtsprdt_stage:2, rtsp_control_flow:1;
 
     /* NDPI_PROTOCOL_YAHOO */
-    u_int32_t yahoo_detection_finished:2;
+    u_int8_t yahoo_detection_finished:2;
 
     /* NDPI_PROTOCOL_ZATTOO */
-    u_int32_t zattoo_stage:3;
+    u_int8_t zattoo_stage:3;
 
     /* NDPI_PROTOCOL_QQ */
-    u_int32_t qq_stage:3;
+    u_int8_t qq_stage:3;
 
     /* NDPI_PROTOCOL_THUNDER */
-    u_int32_t thunder_stage:2;		        // 0 - 3
+    u_int8_t thunder_stage:2; // 0 - 3
 
     /* NDPI_PROTOCOL_OSCAR */
-    u_int32_t oscar_ssl_voice_stage:3;
-    u_int32_t oscar_video_voice:1;
+    u_int8_t oscar_ssl_voice_stage:3, oscar_video_voice:1;
 
     /* NDPI_PROTOCOL_FLORENSIA */
-    u_int32_t florensia_stage:1;
+    u_int8_t florensia_stage:1;
 
     /* NDPI_PROTOCOL_SOCKS */
-    u_int32_t socks5_stage:2;	                // 0 - 3
-    u_int32_t socks4_stage:2;	                // 0 - 3
+    u_int8_t socks5_stage:2, socks4_stage:2; // 0 - 3
 
     /* NDPI_PROTOCOL_EDONKEY */
-    u_int32_t edonkey_stage:2;	                // 0 - 3
+    u_int8_t edonkey_stage:2; // 0 - 3
 
     /* NDPI_PROTOCOL_FTP_CONTROL */
-    u_int32_t ftp_control_stage:2;
+    u_int8_t ftp_control_stage:2;
 
     /* NDPI_PROTOCOL_RTMP */
-    u_int32_t rtmp_stage:2;
+    u_int8_t rtmp_stage:2;
 
     /* NDPI_PROTOCOL_PANDO */
-    u_int32_t pando_stage:3;
+    u_int8_t pando_stage:3;
 
     /* NDPI_PROTOCOL_STEAM */
-    u_int32_t steam_stage:3;
-    u_int32_t steam_stage1:3;			// 0 - 4
-    u_int32_t steam_stage2:2;			// 0 - 2
-    u_int32_t steam_stage3:2;			// 0 - 2
+    u_int16_t steam_stage:3, steam_stage1:3, steam_stage2:2, steam_stage3:2;
 
     /* NDPI_PROTOCOL_PPLIVE */
-    u_int32_t pplive_stage1:3;			// 0 - 6
-    u_int32_t pplive_stage2:2;			// 0 - 2
-    u_int32_t pplive_stage3:2;			// 0 - 2
+    u_int8_t pplive_stage1:3, pplive_stage2:2, pplive_stage3:2;
 
     /* NDPI_PROTOCOL_STARCRAFT */
-    u_int32_t starcraft_udp_stage : 3;	// 0-7
+    u_int8_t starcraft_udp_stage:3; // 0-7
 
     /* NDPI_PROTOCOL_OPENVPN */
     u_int8_t ovpn_session_id[8];
@@ -1125,9 +1156,9 @@ typedef struct {
 } ndpi_network;
 
 typedef struct {
-  int protocol_id;
-  ndpi_protocol_category_t protocol_category;
-  ndpi_protocol_breed_t protocol_breed;
+    int protocol_id;
+    ndpi_protocol_category_t protocol_category;
+    ndpi_protocol_breed_t protocol_breed;
 } ndpi_protocol_match_result;
 
 #endif /* __NDPI_TYPEDEFS_H__ */
