@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2018 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2019 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,13 +22,13 @@
 #include <set>
 #include <stdexcept>
 #include <unordered_map>
+#include <list>
 #include <vector>
 #include <sstream>
 #ifdef HAVE_ATOMIC
 #include <atomic>
-#else
-typedef bool atomic_bool;
 #endif
+#include <regex>
 
 #include <sys/stat.h>
 #include <sys/select.h>
@@ -46,7 +46,9 @@ typedef bool atomic_bool;
 #ifdef _ND_USE_NETLINK
 #include <linux/netlink.h>
 #endif
+
 #include <json.h>
+#include <pcap/pcap.h>
 
 #include <libmnl/libmnl.h>
 #include <libnetfilter_conntrack/libnetfilter_conntrack.h>
@@ -545,15 +547,15 @@ void ndConntrackThread::ClassifyFlow(ndFlow *flow)
     switch (family) {
     case AF_INET:
         sha1_write(&ctx,
-            (const char *)&flow->lower_addr, sizeof(struct in_addr));
+            (const char *)&flow->lower_addr4->sin_addr, sizeof(struct in_addr));
         sha1_write(&ctx,
-            (const char *)&flow->upper_addr, sizeof(struct in_addr));
+            (const char *)&flow->upper_addr4->sin_addr, sizeof(struct in_addr));
         break;
     case AF_INET6:
         sha1_write(&ctx,
-            (const char *)&flow->lower_addr6, sizeof(struct in6_addr));
+            (const char *)&flow->lower_addr6->sin6_addr, sizeof(struct in6_addr));
         sha1_write(&ctx,
-            (const char *)&flow->upper_addr6, sizeof(struct in6_addr));
+            (const char *)&flow->upper_addr6->sin6_addr, sizeof(struct in6_addr));
         break;
     }
 
@@ -591,8 +593,10 @@ void ndConntrackThread::ClassifyFlow(ndFlow *flow)
             }
 #endif
             if (memcmp(sa_orig_src, sa_repl_dst, sizeof(struct sockaddr_in)) ||
-                memcmp(sa_orig_dst, sa_repl_src, sizeof(struct sockaddr_in)))
+                memcmp(sa_orig_dst, sa_repl_src, sizeof(struct sockaddr_in))) {
                 flow->ip_nat = true;
+                ct_flow->updated_at = time(NULL);
+            }
 
             break;
 
@@ -613,8 +617,10 @@ void ndConntrackThread::ClassifyFlow(ndFlow *flow)
             }
 #endif
             if (memcmp(sa6_orig_src, sa6_repl_dst, sizeof(struct sockaddr_in6)) ||
-                memcmp(sa6_orig_dst, sa6_repl_src, sizeof(struct sockaddr_in6)))
+                memcmp(sa6_orig_dst, sa6_repl_src, sizeof(struct sockaddr_in6))) {
                 flow->ip_nat = true;
+                ct_flow->updated_at = time(NULL);
+            }
 
             break;
         }
@@ -654,10 +660,8 @@ void ndConntrackThread::DumpStats(void)
 #endif
 
 ndConntrackFlow::ndConntrackFlow(uint32_t id, struct nf_conntrack *ct)
-    : id(id), created_at(0), l3_proto(0), l4_proto(0)
+    : id(id), updated_at(0), l3_proto(0), l4_proto(0)
 {
-    created_at = time(NULL);
-
     orig_port[ndCT_DIR_SRC] = 0;
     orig_port[ndCT_DIR_DST] = 0;
     repl_port[ndCT_DIR_SRC] = 0;
@@ -668,6 +672,8 @@ ndConntrackFlow::ndConntrackFlow(uint32_t id, struct nf_conntrack *ct)
 
 void ndConntrackFlow::Update(struct nf_conntrack *ct)
 {
+    updated_at = time(NULL);
+
     orig_addr_valid[ndCT_DIR_SRC] = false;
     orig_addr_valid[ndCT_DIR_DST] = false;
     repl_addr_valid[ndCT_DIR_SRC] = false;
