@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2018 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2019 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,8 +21,11 @@
 #include <stdexcept>
 #include <cstring>
 #include <map>
+#include <list>
 #include <vector>
 #include <unordered_map>
+#include <sstream>
+#include <regex>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -32,10 +35,8 @@
 
 #include <arpa/inet.h>
 
-#ifdef _ND_USE_NETLINK
-#include <linux/netlink.h>
-#endif
 #include <json.h>
+#include <pcap/pcap.h>
 
 using namespace std;
 
@@ -47,16 +48,209 @@ using namespace std;
 #endif
 #include "nd-json.h"
 #include "nd-util.h"
+
+// Enable flow hash cache debug logging
+//#define _ND_DEBUG_FHC 1
+
 #include "nd-flow.h"
 
 extern nd_global_config nd_config;
 extern nd_device_ethers device_ethers;
 
-void ndFlow::hash(const string &device, string &digest,
-    bool full_hash, const uint8_t *key, size_t key_length)
+ndFlowHashCache::ndFlowHashCache(const string &device, size_t cache_size)
+    : device(device), cache_size(cache_size) { }
+
+void ndFlowHashCache::push(const string &lower_hash, const string &upper_hash)
+{
+    nd_fhc_map::const_iterator i = lookup.find(lower_hash);
+
+    if (i != lookup.end()) {
+        nd_debug_printf("%s: WARNING: Found existing hash in flow hash cache on push.\n",
+            device.c_str());
+    }
+    else {
+        if (lookup.size() == cache_size) {
+//#if _ND_DEBUG_FHC
+            nd_debug_printf("%s: Purging old flow hash cache entries.\n", device.c_str());
+//#endif
+            for (size_t n = 0; n < cache_size / nd_config.fhc_purge_divisor; n++) {
+                pair<string, string> j = index.back();
+
+                nd_fhc_map::iterator k = lookup.find(j.first);
+                if (k == lookup.end()) {
+                    nd_debug_printf("%s: WARNING: flow hash cache index not found in map\n",
+                        device.c_str());
+                }
+                else
+                    lookup.erase(k);
+
+                index.pop_back();
+            }
+        }
+
+        index.push_front(make_pair(lower_hash, upper_hash));
+        lookup[lower_hash] = index.begin();
+#if _ND_DEBUG_FHC
+        nd_debug_printf("%s: Flow hash cache entries: %lu\n", device.c_str(), lookup.size());
+#endif
+    }
+}
+
+bool ndFlowHashCache::pop(const string &lower_hash, string &upper_hash)
+{
+    nd_fhc_map::iterator i = lookup.find(lower_hash);
+    if (i == lookup.end()) return false;
+
+    upper_hash = i->second->second;
+
+    index.erase(i->second);
+
+    index.push_front(make_pair(lower_hash, upper_hash));
+
+    i->second = index.begin();
+
+    return true;
+}
+
+void ndFlowHashCache::save(void)
+{
+    ostringstream os;
+
+    switch (nd_config.fhc_save) {
+    case ndFHC_PERSISTENT:
+        os << ND_PERSISTENT_STATEDIR << ND_FLOW_HC_FILE_NAME << device << ".dat";
+        break;
+    case ndFHC_VOLATILE:
+        os << ND_VOLATILE_STATEDIR << ND_FLOW_HC_FILE_NAME << device << ".dat";
+        break;
+    default:
+        return;
+    }
+
+    FILE *hf = fopen(os.str().c_str(), "wb");
+    if (hf == NULL) {
+        nd_printf("%s: WARNING: Error saving flow hash cache: %s: %s\n",
+            device.c_str(), os.str().c_str(), strerror(errno));
+        return;
+    }
+
+    nd_fhc_list::iterator i;
+    for (i = index.begin(); i != index.end(); i++) {
+        fwrite((*i).first.c_str(), 1, SHA1_DIGEST_LENGTH, hf);
+        fwrite((*i).second.c_str(), 1, SHA1_DIGEST_LENGTH, hf);
+    }
+    fclose(hf);
+
+    nd_debug_printf("%s: Saved %lu flow hash cache entries.\n",
+        device.c_str(), index.size ());
+}
+
+void ndFlowHashCache::load(void)
+{
+    ostringstream os;
+
+    switch (nd_config.fhc_save) {
+    case ndFHC_PERSISTENT:
+        os << ND_PERSISTENT_STATEDIR << ND_FLOW_HC_FILE_NAME << device << ".dat";
+        break;
+    case ndFHC_VOLATILE:
+        os << ND_VOLATILE_STATEDIR << ND_FLOW_HC_FILE_NAME << device << ".dat";
+        break;
+    default:
+        return;
+    }
+
+    FILE *hf = fopen(os.str().c_str(), "rb");
+    if (hf != NULL) {
+        do {
+            string digest_lower, digest_mdata;
+            uint8_t digest[SHA1_DIGEST_LENGTH * 2];
+
+            if (fread(digest, SHA1_DIGEST_LENGTH * 2, 1, hf) != 1) break;
+
+            digest_lower.assign((const char *)digest, SHA1_DIGEST_LENGTH);
+            digest_mdata.assign((const char *)&digest[SHA1_DIGEST_LENGTH],
+                SHA1_DIGEST_LENGTH);
+
+            push(digest_lower, digest_mdata);
+        }
+        while (! feof(hf));
+
+        fclose(hf);
+    }
+
+    nd_debug_printf("%s: Loaded %lu flow hash cache entries.\n",
+        device.c_str(), index.size());
+}
+
+ndFlow::ndFlow(bool internal)
+    : internal(internal), ip_version(0), ip_protocol(0), vlan_id(0),
+    ip_nat(false), tcp_fin(false),
+    ts_first_seen(0), ts_first_update(0), ts_last_seen(0),
+    lower_port(0), upper_port(0),
+    lower_bytes(0), upper_bytes(0), total_bytes(0),
+    lower_packets(0), upper_packets(0), total_packets(0),
+    detection_complete(false), detection_guessed(0),
+    ndpi_flow(NULL), id_src(NULL), id_dst(NULL),
+    privacy_mask(0), origin(0)
+{
+    memset(lower_mac, 0, ETH_ALEN);
+    memset(upper_mac, 0, ETH_ALEN);
+
+    memset(&lower_addr, 0, sizeof(struct sockaddr_storage));
+    memset(&upper_addr, 0, sizeof(struct sockaddr_storage));
+
+    lower_addr4 = (struct sockaddr_in *)&lower_addr;
+    lower_addr6 = (struct sockaddr_in6 *)&lower_addr;
+    upper_addr4 = (struct sockaddr_in *)&upper_addr;
+    upper_addr6 = (struct sockaddr_in6 *)&upper_addr;
+
+    memset(lower_ip, 0, INET6_ADDRSTRLEN);
+    memset(upper_ip, 0, INET6_ADDRSTRLEN);
+
+    memset(&detected_protocol, 0, sizeof(ndpi_protocol));
+
+    memset(digest_lower, 0, SHA1_DIGEST_LENGTH);
+    memset(digest_mdata, 0, SHA1_DIGEST_LENGTH);
+
+    memset(host_server_name, 0, ND_MAX_HOSTNAME);
+
+    memset(http.user_agent, 0, ND_FLOW_UA_LEN);
+    memset(http.url, 0, ND_FLOW_URL_LEN);
+
+    memset(dhcp.fingerprint, 0, ND_FLOW_DHCPFP_LEN);
+    memset(dhcp.class_ident, 0, ND_FLOW_DHCPCI_LEN);
+
+    memset(ssh.client_agent, 0, ND_FLOW_SSH_UALEN);
+    memset(ssh.server_agent, 0, ND_FLOW_SSH_UALEN);
+
+    ssl.version = 0;
+    ssl.cipher_suite = 0;
+    memset(ssl.client_sni, 0, ND_FLOW_SSL_CNLEN);
+    memset(ssl.server_cn, 0, ND_FLOW_SSL_CNLEN);
+    memset(ssl.server_organization, 0, ND_FLOW_SSL_ORGLEN);
+    memset(ssl.client_ja3, 0, ND_FLOW_SSL_JA3LEN);
+    memset(ssl.server_ja3, 0, ND_FLOW_SSL_JA3LEN);
+
+    smtp.tls = false;
+
+    bt.info_hash_valid = 0;
+    memset(bt.info_hash, 0, ND_FLOW_BTIHASH_LEN);
+
+    memset(mdns.answer, 0, ND_FLOW_MDNS_ANSLEN);
+
+    memset(capture_filename, 0, sizeof(ND_FLOW_CAPTURE_TEMPLATE));
+}
+
+ndFlow::~ndFlow()
+{
+    release();
+}
+
+void ndFlow::hash(const string &device,
+    bool hash_mdata, const uint8_t *key, size_t key_length)
 {
     sha1 ctx;
-    uint8_t _digest[SHA1_DIGEST_LENGTH];
 
     sha1_init(&ctx);
     sha1_write(&ctx, (const char *)device.c_str(), device.size());
@@ -70,12 +264,16 @@ void ndFlow::hash(const string &device, string &digest,
 
     switch (ip_version) {
     case 4:
-        sha1_write(&ctx, (const char *)&lower_addr, sizeof(struct in_addr));
-        sha1_write(&ctx, (const char *)&upper_addr, sizeof(struct in_addr));
+        sha1_write(&ctx,
+            (const char *)&lower_addr4->sin_addr, sizeof(struct in_addr));
+        sha1_write(&ctx,
+            (const char *)&upper_addr4->sin_addr, sizeof(struct in_addr));
         break;
     case 6:
-        sha1_write(&ctx, (const char *)&lower_addr6, sizeof(struct in6_addr));
-        sha1_write(&ctx, (const char *)&upper_addr6, sizeof(struct in6_addr));
+        sha1_write(&ctx,
+            (const char *)&lower_addr6->sin6_addr, sizeof(struct in6_addr));
+        sha1_write(&ctx,
+            (const char *)&upper_addr6->sin6_addr, sizeof(struct in6_addr));
         break;
     default:
         break;
@@ -84,7 +282,7 @@ void ndFlow::hash(const string &device, string &digest,
     sha1_write(&ctx, (const char *)&lower_port, sizeof(lower_port));
     sha1_write(&ctx, (const char *)&upper_port, sizeof(upper_port));
 
-    if (full_hash) {
+    if (hash_mdata) {
         sha1_write(&ctx,
             (const char *)&detection_guessed, sizeof(detection_guessed));
         sha1_write(&ctx,
@@ -92,15 +290,15 @@ void ndFlow::hash(const string &device, string &digest,
 
         if (host_server_name[0] != '\0') {
             sha1_write(&ctx,
-                host_server_name, strnlen(host_server_name, HOST_NAME_MAX));
+                host_server_name, strnlen(host_server_name, ND_MAX_HOSTNAME));
         }
-        if (has_ssl_client_certcn()) {
+        if (has_ssl_client_sni()) {
             sha1_write(&ctx,
-                ssl.client_certcn, strnlen(ssl.client_certcn, ND_FLOW_SSL_CNLEN));
+                ssl.client_sni, strnlen(ssl.client_sni, ND_FLOW_SSL_CNLEN));
         }
-        if (has_ssl_server_certcn()) {
+        if (has_ssl_server_cn()) {
             sha1_write(&ctx,
-                ssl.server_certcn, strnlen(ssl.server_certcn, ND_FLOW_SSL_CNLEN));
+                ssl.server_cn, strnlen(ssl.server_cn, ND_FLOW_SSL_CNLEN));
         }
         if (has_bt_info_hash()) {
             sha1_write(&ctx, bt.info_hash, ND_FLOW_BTIHASH_LEN);
@@ -113,7 +311,74 @@ void ndFlow::hash(const string &device, string &digest,
     if (key != NULL && key_length > 0)
         sha1_write(&ctx, (const char *)key, key_length);
 
-    digest.assign((const char *)sha1_result(&ctx, _digest), SHA1_DIGEST_LENGTH);
+    if (! hash_mdata)
+        sha1_result(&ctx, digest_lower);
+    else
+        sha1_result(&ctx, digest_mdata);
+}
+
+void ndFlow::push(const struct pcap_pkthdr *pkt_header, const uint8_t *pkt_data)
+{
+    struct pcap_pkthdr *header = new struct pcap_pkthdr;
+    if (header == NULL)
+        throw ndSystemException(__PRETTY_FUNCTION__, "new header", ENOMEM);
+    uint8_t *data = new uint8_t[pkt_header->len];
+    if (data == NULL)
+        throw ndSystemException(__PRETTY_FUNCTION__, "new data", ENOMEM);
+
+    memcpy(header, pkt_header, sizeof(struct pcap_pkthdr));
+    memcpy(data, pkt_data, pkt_header->caplen);
+
+    capture.push_back(make_pair(header, data));
+}
+
+int ndFlow::dump(pcap_t *pcap, const uint8_t *digest)
+{
+    char *p = capture_filename;
+    memcpy(p, ND_FLOW_CAPTURE_TEMPLATE, sizeof(ND_FLOW_CAPTURE_TEMPLATE));
+
+    p += ND_FLOW_CAPTURE_SUB_OFFSET;
+    for (int i = 0; i < 4; i++, p += 2) sprintf(p, "%02hhx", digest[i]);
+    strcat(p, ".cap");
+
+    pcap_dumper_t *pcap_dumper = pcap_dump_open(pcap, capture_filename);
+
+    if (pcap_dumper == NULL) {
+        nd_debug_printf("%s: pcap_dump_open: %s: %s\n",
+            __PRETTY_FUNCTION__, capture_filename, "unknown");
+        return -1;
+    }
+
+    for (nd_flow_capture::const_iterator i = capture.begin();
+        i != capture.end(); i++) {
+        pcap_dump((uint8_t *)pcap_dumper, i->first, i->second);
+    }
+
+    pcap_dump_close(pcap_dumper);
+
+    return 0;
+}
+
+void ndFlow::reset(void)
+{
+    ts_first_update = 0;
+    lower_bytes = upper_bytes = 0;
+    lower_packets = upper_packets = 0;
+}
+
+void ndFlow::release(void)
+{
+    if (ndpi_flow != NULL) { ndpi_free_flow(ndpi_flow); ndpi_flow = NULL; }
+    if (id_src != NULL) { delete id_src; id_src = NULL; }
+    if (id_dst != NULL) { delete id_dst; id_dst = NULL; }
+
+    for (nd_flow_capture::const_iterator i = capture.begin();
+        i != capture.end(); i++) {
+        delete i->first;
+        delete [] i->second;
+    }
+
+    capture.clear();
 }
 
 uint16_t ndFlow::master_protocol(void)
@@ -125,6 +390,7 @@ uint16_t ndFlow::master_protocol(void)
 
     switch (proto) {
     case NDPI_PROTOCOL_GMAIL:
+    case NDPI_PROTOCOL_HTTPS:
     case NDPI_PROTOCOL_MAIL_IMAP:
     case NDPI_PROTOCOL_MAIL_IMAPS:
     case NDPI_PROTOCOL_MAIL_POPS:
@@ -179,6 +445,13 @@ bool ndFlow::has_http_user_agent(void)
     );
 }
 
+bool ndFlow::has_http_url(void)
+{
+    return (
+        http.url[0] != '\0'
+    );
+}
+
 bool ndFlow::has_ssh_client_agent(void)
 {
     return (
@@ -197,19 +470,43 @@ bool ndFlow::has_ssh_server_agent(void)
     );
 }
 
-bool ndFlow::has_ssl_client_certcn(void)
+bool ndFlow::has_ssl_client_sni(void)
 {
     return (
         master_protocol() == NDPI_PROTOCOL_SSL &&
-        ssl.client_certcn[0] != '\0'
+        ssl.client_sni[0] != '\0'
     );
 }
 
-bool ndFlow::has_ssl_server_certcn(void)
+bool ndFlow::has_ssl_server_cn(void)
 {
     return (
         master_protocol() == NDPI_PROTOCOL_SSL &&
-        ssl.server_certcn[0] != '\0'
+        ssl.server_cn[0] != '\0'
+    );
+}
+
+bool ndFlow::has_ssl_server_organization(void)
+{
+    return (
+        master_protocol() == NDPI_PROTOCOL_SSL &&
+        ssl.server_organization[0] != '\0'
+    );
+}
+
+bool ndFlow::has_ssl_client_ja3(void)
+{
+    return (
+        master_protocol() == NDPI_PROTOCOL_SSL &&
+        ssl.client_ja3[0] != '\0'
+    );
+}
+
+bool ndFlow::has_ssl_server_ja3(void)
+{
+    return (
+        master_protocol() == NDPI_PROTOCOL_SSL &&
+        ssl.server_ja3[0] != '\0'
     );
 }
 
@@ -285,11 +582,11 @@ void ndFlow::print(const char *tag, struct ndpi_detection_module_struct *ndpi)
         (host_server_name[0] != '\0' || has_mdns_answer()) ? " H: " : "",
         (host_server_name[0] != '\0' || has_mdns_answer()) ?
             has_mdns_answer() ? mdns.answer : host_server_name : "",
-        (has_ssl_client_certcn() || has_ssl_server_certcn()) ? " SSL" : "",
-        (has_ssl_client_certcn()) ? " C: " : "",
-        (has_ssl_client_certcn()) ? ssl.client_certcn : "",
-        (has_ssl_server_certcn()) ? " S: " : "",
-        (has_ssl_server_certcn()) ? ssl.server_certcn : "",
+        (has_ssl_client_sni() || has_ssl_server_cn()) ? " SSL" : "",
+        (has_ssl_client_sni()) ? " C: " : "",
+        (has_ssl_client_sni()) ? ssl.client_sni : "",
+        (has_ssl_server_cn()) ? " S: " : "",
+        (has_ssl_server_cn()) ? ssl.server_cn : "",
         (has_bt_info_hash()) ? " BT-IH: " : "",
         (has_bt_info_hash()) ? digest.c_str() : ""
     );
@@ -301,8 +598,8 @@ void ndFlow::print(const char *tag, struct ndpi_detection_module_struct *ndpi)
     }
 }
 
-json_object *ndFlow::json_encode(const string &device,
-    ndJson &json, struct ndpi_detection_module_struct *ndpi, bool include_stats)
+json_object *ndFlow::json_encode(ndJson &json,
+    struct ndpi_detection_module_struct *ndpi, bool include_stats)
 {
     char mac_addr[ND_STR_ETHALEN + 1];
     string other_type = "unknown";
@@ -314,9 +611,8 @@ json_object *ndFlow::json_encode(const string &device,
 
     json_object *json_flow = json.CreateObject();
 
-    string digest, digest_bin;
-    hash(device, digest_bin, true);
-    nd_sha1_to_string((const uint8_t *)digest_bin.c_str(), digest);
+    string digest;
+    nd_sha1_to_string(digest_mdata, digest);
     json.AddObject(json_flow, "digest", digest);
 
     json.AddObject(json_flow, "ip_nat", ip_nat);
@@ -447,7 +743,9 @@ json_object *ndFlow::json_encode(const string &device,
     // Spend some time to figure out if there are any
     // possible over-matches for different methods of
     // deployment (gateway/port mirror modes).
-    else if (lower_type == ndNETLINK_ATYPE_PRIVATE &&
+#endif
+    else if (ip_version != 6 &&
+        lower_type == ndNETLINK_ATYPE_PRIVATE &&
         upper_type == ndNETLINK_ATYPE_PRIVATE) {
         other_type = "local";
         _lower_mac = "local_mac";
@@ -461,7 +759,6 @@ json_object *ndFlow::json_encode(const string &device,
         _upper_bytes = "other_bytes";
         _upper_packets = "other_packets";
     }
-#endif
     else if (lower_type == ndNETLINK_ATYPE_PRIVATE &&
         upper_type == ndNETLINK_ATYPE_LOCALIP) {
         other_type = "remote";
@@ -554,10 +851,12 @@ json_object *ndFlow::json_encode(const string &device,
         break;
     }
 
+    // 00-52-14 to 00-52-FF: Unassigned (small allocations)
     if (privacy_mask & PRIVATE_LOWER)
-        snprintf(mac_addr, sizeof(mac_addr), "01:02:03:04:05:06");
+        snprintf(mac_addr, sizeof(mac_addr), "00:52:14:00:00:00");
     else {
-        snprintf(mac_addr, sizeof(mac_addr), "%02x:%02x:%02x:%02x:%02x:%02x",
+        snprintf(mac_addr, sizeof(mac_addr),
+            "%02hhx:%02hhx:%02hhx:%02hhx:%02hhx:%02hhx",
             lower_mac[0], lower_mac[1], lower_mac[2],
             lower_mac[3], lower_mac[4], lower_mac[5]
         );
@@ -565,9 +864,10 @@ json_object *ndFlow::json_encode(const string &device,
     json.AddObject(json_flow, _lower_mac, mac_addr);
 
     if (privacy_mask & PRIVATE_UPPER)
-        snprintf(mac_addr, sizeof(mac_addr), "0a:0b:0c:0d:0e:0f");
+        snprintf(mac_addr, sizeof(mac_addr), "00:52:FF:00:00:00");
     else {
-        snprintf(mac_addr, sizeof(mac_addr), "%02x:%02x:%02x:%02x:%02x:%02x",
+        snprintf(mac_addr, sizeof(mac_addr),
+            "%02hhx:%02hhx:%02hhx:%02hhx:%02hhx:%02hhx",
             upper_mac[0], upper_mac[1], upper_mac[2],
             upper_mac[3], upper_mac[4], upper_mac[5]
         );
@@ -576,18 +876,18 @@ json_object *ndFlow::json_encode(const string &device,
 
     if (privacy_mask & PRIVATE_LOWER) {
         if (ip_version == 4)
-            json.AddObject(json_flow, _lower_ip, "1.2.3.1");
+            json.AddObject(json_flow, _lower_ip, ND_PRIVATE_IPV4 "253");
         else
-            json.AddObject(json_flow, _lower_ip, "1230::1");
+            json.AddObject(json_flow, _lower_ip, ND_PRIVATE_IPV6 "fd");
     }
     else
         json.AddObject(json_flow, _lower_ip, lower_ip);
 
     if (privacy_mask & PRIVATE_UPPER) {
         if (ip_version == 4)
-            json.AddObject(json_flow, _upper_ip, "1.2.3.2");
+            json.AddObject(json_flow, _upper_ip, ND_PRIVATE_IPV4 "254");
         else
-            json.AddObject(json_flow, _upper_ip, "1230::2");
+            json.AddObject(json_flow, _upper_ip, ND_PRIVATE_IPV6 "fe");
     }
     else
         json.AddObject(json_flow, _upper_ip, upper_ip);
@@ -609,9 +909,9 @@ json_object *ndFlow::json_encode(const string &device,
     json.AddObject(json_flow, "detected_protocol_name",
         ndpi_get_proto_name(ndpi, detected_protocol.master_protocol));
 
-    json.AddObject(json_flow, "detected_service",
+    json.AddObject(json_flow, "detected_application",
         (int32_t)detected_protocol.app_protocol);
-    json.AddObject(json_flow, "detected_service_name",
+    json.AddObject(json_flow, "detected_application_name",
         ndpi_get_proto_name(ndpi, detected_protocol.app_protocol));
 
     json.AddObject(json_flow, "detection_guessed", detection_guessed);
@@ -621,11 +921,14 @@ json_object *ndFlow::json_encode(const string &device,
             "host_server_name", host_server_name);
     }
 
-    if (has_http_user_agent()) {
+    if (has_http_user_agent() || has_http_url()) {
 
         json_object *_http = json.CreateObject(json_flow, "http");
 
-        json.AddObject(_http, "user_agent", http.user_agent);
+        if (has_http_user_agent())
+            json.AddObject(_http, "user_agent", http.user_agent);
+        if (has_http_url())
+            json.AddObject(_http, "url", http.url);
     }
 
     if (has_dhcp_fingerprint() || has_dhcp_class_ident()) {
@@ -650,7 +953,7 @@ json_object *ndFlow::json_encode(const string &device,
             json.AddObject(_ssh, "server", ssh.server_agent);
     }
 
-    if (has_ssl_client_certcn() || has_ssl_server_certcn()) {
+    if (has_ssl_client_sni() || has_ssl_server_cn()) {
 
         char tohex[7];
         json_object *_ssl = json.CreateObject(json_flow, "ssl");
@@ -661,11 +964,20 @@ json_object *ndFlow::json_encode(const string &device,
         sprintf(tohex, "0x%04hx", ssl.cipher_suite);
         json.AddObject(_ssl, "cipher_suite", tohex);
 
-        if (has_ssl_client_certcn())
-            json.AddObject(_ssl, "client", ssl.client_certcn);
+        if (has_ssl_client_sni())
+            json.AddObject(_ssl, "client_sni", ssl.client_sni);
 
-        if (has_ssl_server_certcn())
-            json.AddObject(_ssl, "server", ssl.server_certcn);
+        if (has_ssl_server_cn())
+            json.AddObject(_ssl, "server_cn", ssl.server_cn);
+
+        if (has_ssl_server_organization())
+            json.AddObject(_ssl, "organization", ssl.server_organization);
+
+        if (has_ssl_client_ja3())
+            json.AddObject(_ssl, "client_ja3", ssl.client_ja3);
+
+        if (has_ssl_server_ja3())
+            json.AddObject(_ssl, "server_ja3", ssl.server_ja3);
     }
 
     if (has_bt_info_hash()) {
@@ -684,6 +996,7 @@ json_object *ndFlow::json_encode(const string &device,
     }
 
     json.AddObject(json_flow, "first_seen_at", ts_first_seen);
+    json.AddObject(json_flow, "first_update_at", ts_first_update);
     json.AddObject(json_flow, "last_seen_at", ts_last_seen);
 
     return json_flow;
