@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2018 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2019 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,24 +18,34 @@
 #include "config.h"
 #endif
 
+#include <map>
+#include <unordered_map>
+#include <vector>
 #include <string>
 #include <sstream>
 #include <stdexcept>
 #include <deque>
 #include <vector>
 #include <map>
+#include <regex>
 
-#include <unistd.h>
 #include <sys/inotify.h>
 #include <sys/stat.h>
+#include <sys/socket.h>
+
+#include <unistd.h>
 #include <fcntl.h>
 #include <signal.h>
+#include <string.h>
 #include <errno.h>
 
 #include <json.h>
+#include <pcap/pcap.h>
 
 using namespace std;
 
+#include "netifyd.h"
+
 #include "nd-ndpi.h"
 #include "nd-sha1.h"
 #include "nd-json.h"
@@ -84,7 +94,7 @@ void ndInotify::AddWatch(const string &tag, const string &filename)
         watch->wd = -1;
         watch->filename = filename.c_str();
 
-        inotify_watch[filename] = watch;
+        inotify_watch[tag] = watch;
     }
 }
 
