@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2018 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2019 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,14 +25,14 @@
 #include <stdexcept>
 #ifdef HAVE_ATOMIC
 #include <atomic>
-#else
-typedef bool atomic_bool;
 #endif
+#include <regex>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <pthread.h>
+#include <errno.h>
 
 #include <sys/stat.h>
 #include <sys/un.h>
@@ -40,6 +40,7 @@ typedef bool atomic_bool;
 #include <arpa/inet.h>
 
 #include <json.h>
+#include <pcap/pcap.h>
 
 using namespace std;
 
@@ -119,7 +120,7 @@ struct ndpi_detection_module_struct *ndpi_get_parent(void)
 }
 
 struct ndpi_detection_module_struct *nd_ndpi_init(
-    const string &tag, uint32_t &custom_proto_base)
+    const string &tag __attribute__((unused)), uint32_t &custom_proto_base)
 {
     struct ndpi_detection_module_struct *ndpi = NULL;
 
@@ -132,7 +133,7 @@ struct ndpi_detection_module_struct *nd_ndpi_init(
 
     // Set nDPI preferences
     ndpi_set_detection_preferences(ndpi, ndpi_pref_http_dont_dissect_response, 0);
-    ndpi_set_detection_preferences(ndpi, ndpi_pref_dns_dissect_response, 1);
+    ndpi_set_detection_preferences(ndpi, ndpi_pref_dns_dont_dissect_response, 0);
     ndpi_set_detection_preferences(ndpi, ndpi_pref_direction_detect_disable, 0);
     ndpi_set_detection_preferences(ndpi, ndpi_pref_disable_metadata_export, 0);
     ndpi_set_detection_preferences(ndpi, ndpi_pref_enable_category_substring_match, 0);
