@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2018 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2019 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include <map>
 #include <unordered_map>
 #include <stdexcept>
+#include <regex>
 
 #include <unistd.h>
 #include <string.h>
@@ -34,11 +35,14 @@
 #include <sys/stat.h>
 #include <sys/socket.h>
 
+#ifdef HAVE_LINUX_NETLINK_H
 #include <net/if.h>
 #include <linux/netlink.h>
 #include <linux/rtnetlink.h>
+#endif
 
 #include <json.h>
+#include <pcap/pcap.h>
 
 using namespace std;
 
@@ -114,10 +118,12 @@ inline bool ndNetlinkNetworkAddr::operator!=(const ndNetlinkNetworkAddr &n) cons
 
     return (rc != 0);
 }
-
 ndNetlink::ndNetlink(const nd_ifaces &ifaces)
+#ifdef HAVE_LINUX_NETLINK_H
     : nd(-1), seq(0)
+#endif
 {
+#ifdef HAVE_LINUX_NETLINK_H
     int rc;
 
     memset(buffer, 0, ND_NETLINK_BUFSIZ);
@@ -161,15 +167,17 @@ ndNetlink::ndNetlink(const nd_ifaces &ifaces)
         nd_printf("Error setting netlink socket flags: %s\n", strerror(rc));
         throw ndNetlinkException(strerror(rc));
     }
-
+#endif
     for (nd_ifaces::const_iterator i = ifaces.begin(); i != ifaces.end(); i++)
         AddInterface((*i).second);
 
     // Add private networks for when all else fails...
+    AddNetwork(AF_INET, _ND_NETLINK_PRIVATE, "127.0.0.0", 8);
     AddNetwork(AF_INET, _ND_NETLINK_PRIVATE, "10.0.0.0", 8);
     AddNetwork(AF_INET, _ND_NETLINK_PRIVATE, "172.16.0.0", 12);
     AddNetwork(AF_INET, _ND_NETLINK_PRIVATE, "192.168.0.0", 16);
     AddNetwork(AF_INET6, _ND_NETLINK_PRIVATE, "fc00::", 7);
+    AddNetwork(AF_INET6, _ND_NETLINK_PRIVATE, "fe80::", 10);
 
     // Add multicast networks
     AddNetwork(AF_INET, _ND_NETLINK_MULTICAST, "224.0.0.0", 4);
@@ -183,7 +191,9 @@ ndNetlink::ndNetlink(const nd_ifaces &ifaces)
 
 ndNetlink::~ndNetlink()
 {
+#ifdef HAVE_LINUX_NETLINK_H
     if (nd >= 0) close(nd);
+#endif
     for (ndNetlinkInterfaces::const_iterator i = ifaces.begin();
         i != ifaces.end(); i++) {
         if (i->second != NULL) {
@@ -192,7 +202,7 @@ ndNetlink::~ndNetlink()
         }
     }
 }
-
+#ifdef HAVE_LINUX_NETLINK_H
 void ndNetlink::Refresh(void)
 {
     int rc;
@@ -302,6 +312,7 @@ bool ndNetlink::ProcessEvent(void)
 #endif
     return (added_net || removed_net || added_addr || removed_addr) ? true : false;
 }
+#endif // HAVE_LINUX_NETLINK_H
 
 ndNetlinkAddressType ndNetlink::ClassifyAddress(
     const struct sockaddr_storage *addr)
@@ -572,6 +583,7 @@ bool ndNetlink::AddInterface(const string &iface)
     return true;
 }
 
+#ifdef HAVE_LINUX_NETLINK_H
 bool ndNetlink::ParseMessage(struct rtmsg *rtm, size_t offset,
     string &iface, ndNetlinkNetworkAddr &addr)
 {
@@ -818,6 +830,7 @@ bool ndNetlink::AddNetwork(struct nlmsghdr *nlh)
 
     return true;
 }
+#endif // HAVE_LINUX_NETLINK_H
 
 bool ndNetlink::AddNetwork(sa_family_t family,
     const string &type, const string &saddr, uint8_t length)
@@ -852,6 +865,7 @@ bool ndNetlink::AddNetwork(sa_family_t family,
     return true;
 }
 
+#ifdef HAVE_LINUX_NETLINK_H
 bool ndNetlink::RemoveNetwork(struct nlmsghdr *nlh)
 {
     string iface;
@@ -928,6 +942,7 @@ bool ndNetlink::AddAddress(struct nlmsghdr *nlh)
 
     return true;
 }
+#endif // HAVE_LINUX_NETLINK_H
 
 bool ndNetlink::AddAddress(
     sa_family_t family, const string &type, const string &saddr)
@@ -976,6 +991,7 @@ bool ndNetlink::AddAddress(const string &type, const struct sockaddr_storage &ad
     return true;
 }
 
+#ifdef HAVE_LINUX_NETLINK_H
 bool ndNetlink::RemoveAddress(struct nlmsghdr *nlh)
 {
     string iface;
@@ -1011,6 +1027,7 @@ bool ndNetlink::RemoveAddress(struct nlmsghdr *nlh)
 
     return removed;
 }
+#endif // HAVE_LINUX_NETLINK_H
 
 #ifndef _ND_LEAN_AND_MEAN
 void ndNetlink::Dump(void)
