@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2018 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2019 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -32,9 +32,8 @@
 #include <sstream>
 #ifdef HAVE_ATOMIC
 #include <atomic>
-#else
-typedef bool atomic_bool;
 #endif
+#include <regex>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -43,10 +42,12 @@ typedef bool atomic_bool;
 #include <signal.h>
 #include <pthread.h>
 #include <fcntl.h>
+#include <string.h>
 
 #include <curl/curl.h>
 
 #include <json.h>
+#include <pcap/pcap.h>
 
 #include <zlib.h>
 
@@ -59,11 +60,12 @@ using namespace std;
 #include "nd-thread.h"
 #include "nd-util.h"
 #include "nd-sink.h"
+#include "nd-signal.h"
 
 extern nd_global_config nd_config;
 
-static int nd_curl_debug(
-    CURL *ch, curl_infotype type, char *data, size_t size, void *param)
+static int nd_curl_debug(CURL *ch __attribute__((unused)),
+    curl_infotype type, char *data, size_t size, void *param)
 {
     string buffer;
     if (! ND_DEBUG_UPLOAD) return 0;
@@ -115,10 +117,12 @@ static size_t ndSinkThread_read_data(
 
 #if (LIBCURL_VERSION_NUM < 0x073200)
 static int ndSinkThread_progress(void *user,
-    double dltotal, double dlnow, double ultotal, double ulnow)
+    double dltotal __attribute__((unused)), double dlnow __attribute__((unused)),
+    double ultotal __attribute__((unused)), double ulnow __attribute__((unused)))
 #else
 static int ndSinkThread_progress(void *user,
-    curl_off_t dltotal, curl_off_t dlnow, curl_off_t ultotal, curl_off_t ulnow)
+    curl_off_t dltotal __attribute__((unused)), curl_off_t dlnow __attribute__((unused)),
+    curl_off_t ultotal __attribute__((unused)), curl_off_t ulnow __attribute__((unused)))
 #endif
 {
     ndSinkThread *thread_upload = reinterpret_cast<ndSinkThread *>(user);
@@ -130,18 +134,19 @@ static int ndSinkThread_progress(void *user,
 
 ndSinkThread::ndSinkThread()
     : ndThread("nd-sink", -1),
-    headers(NULL), headers_gz(NULL), pending_size(0)
+    headers(NULL), headers_gz(NULL), pending_size(0), post_errors(0)
 {
     int rc;
 
     if ((ch = curl_easy_init()) == NULL)
         throw ndSinkThreadException("curl_easy_init");
 
-    curl_easy_setopt(ch, CURLOPT_URL, nd_config.url_upload);
+    curl_easy_setopt(ch, CURLOPT_URL, nd_config.url_sink);
     curl_easy_setopt(ch, CURLOPT_POST, 1L);
     curl_easy_setopt(ch, CURLOPT_POSTREDIR, 3L);
     curl_easy_setopt(ch, CURLOPT_FOLLOWLOCATION, 1L);
-    curl_easy_setopt(ch, CURLOPT_TIMEOUT, (long)nd_config.upload_timeout);
+    curl_easy_setopt(ch, CURLOPT_CONNECTTIMEOUT, (long)nd_config.sink_connect_timeout);
+    curl_easy_setopt(ch, CURLOPT_TIMEOUT, (long)nd_config.sink_xfer_timeout);
     curl_easy_setopt(ch, CURLOPT_NOSIGNAL, 1L);
     curl_easy_setopt(ch, CURLOPT_COOKIEFILE, (ND_DEBUG_UPLOAD) ? ND_COOKIE_JAR : "");
 
@@ -156,11 +161,13 @@ ndSinkThread::ndSinkThread()
     curl_easy_setopt(ch, CURLOPT_XFERINFOFUNCTION, ndSinkThread_progress);
     curl_easy_setopt(ch, CURLOPT_XFERINFODATA, static_cast<void *>(this));
 #endif
+#ifdef _ND_WITH_LIBCURL_ZLIB
 #if (LIBCURL_VERSION_NUM < 0x072106)
     curl_easy_setopt(ch, CURLOPT_ENCODING, "gzip");
 #else
     curl_easy_setopt(ch, CURLOPT_ACCEPT_ENCODING, "gzip");
 #endif
+#endif // _ND_WITH_LIBCURL_ZLIB
     if (ND_DEBUG_UPLOAD) {
         curl_easy_setopt(ch, CURLOPT_VERBOSE, 1L);
         curl_easy_setopt(ch, CURLOPT_DEBUGFUNCTION, nd_curl_debug);
@@ -195,10 +202,7 @@ ndSinkThread::ndSinkThread()
 
 ndSinkThread::~ndSinkThread()
 {
-    int rc;
-
-    if ((rc = pthread_cond_broadcast(&uploads_cond)) != 0)
-        throw ndSinkThreadException(strerror(rc));
+    pthread_cond_broadcast(&uploads_cond);
 
     Join();
 
@@ -226,12 +230,10 @@ void *ndSinkThread::Entry(void)
     nd_debug_printf("%s: thread started.\n", tag.c_str());
 
     while (terminate == false) {
-        if ((rc = pthread_mutex_lock(&lock)) != 0)
-            throw ndSinkThreadException(strerror(rc));
+        Lock();
 
         if (uploads.size() == 0) {
-            if ((rc = pthread_mutex_unlock(&lock)) != 0)
-                throw ndSinkThreadException(strerror(rc));
+            Unlock();
 
             if ((rc = pthread_mutex_lock(&uploads_cond_mutex)) != 0)
                 throw ndSinkThreadException(strerror(rc));
@@ -259,8 +261,7 @@ void *ndSinkThread::Entry(void)
         }
         while (uploads.size() > 0);
 
-        if ((rc = pthread_mutex_unlock(&lock)) != 0)
-            throw ndSinkThreadException(strerror(rc));
+        Unlock();
 
         if (terminate == false && pending.size() > 0) Upload();
     }
@@ -272,44 +273,39 @@ void ndSinkThread::Terminate(void)
 {
     int rc;
 
-    if ((rc = pthread_mutex_lock(&lock)) != 0)
-        throw ndSinkThreadException(strerror(rc));
+    Lock();
+
     if ((rc = pthread_cond_broadcast(&uploads_cond)) != 0)
         throw ndSinkThreadException(strerror(rc));
 
     terminate = true;
 
-    if ((rc = pthread_mutex_unlock(&lock)) != 0)
-        throw ndSinkThreadException(strerror(rc));
+    Unlock();
 }
 
 void ndSinkThread::QueuePush(const string &json)
 {
     int rc;
 
-    if ((rc = pthread_mutex_lock(&lock)) != 0)
-        throw ndSinkThreadException(strerror(rc));
+    Lock();
 
     uploads.push(json);
 
     if ((rc = pthread_cond_broadcast(&uploads_cond)) != 0)
         throw ndSinkThreadException(strerror(rc));
-    if ((rc = pthread_mutex_unlock(&lock)) != 0)
-        throw ndSinkThreadException(strerror(rc));
+
+    Unlock();
 }
 
 size_t ndSinkThread::QueuePendingSize(void)
 {
-    int rc;
     size_t bytes;
 
-    if ((rc = pthread_mutex_lock(&lock)) != 0)
-        throw ndSinkThreadException(strerror(rc));
+    Lock();
 
     bytes = pending_size;
 
-    if ((rc = pthread_mutex_unlock(&lock)) != 0)
-        throw ndSinkThreadException(strerror(rc));
+    Unlock();
 
     return bytes;
 }
@@ -322,7 +318,7 @@ void ndSinkThread::PushResponse(ndJsonResponse *response)
 
     pthread_mutex_unlock(&response_mutex);
 
-    kill(getpid(), SIGALRM);
+    kill(getpid(), ND_SIG_SINK_REPLY);
 }
 
 ndJsonResponse *ndSinkThread::PopResponse(void)
@@ -359,18 +355,26 @@ void ndSinkThread::CreateHeaders(void)
             uuid << "X-UUID: " << nd_config.uuid;
     }
 
-    ostringstream serial;
-    serial << "X-UUID-Serial: " << nd_config.uuid_serial;
+    ostringstream uuid_serial;
+    if (strncmp(nd_config.uuid_serial, ND_AGENT_SERIAL_NULL, ND_AGENT_SERIAL_LEN))
+        uuid_serial << "X-UUID-Serial: " << nd_config.uuid_serial;
+    else {
+        string _uuid;
+        if (nd_load_uuid(_uuid, nd_config.path_uuid_serial, ND_AGENT_SERIAL_LEN))
+            uuid_serial << "X-UUID-Serial: " << _uuid;
+        else
+            uuid_serial << "X-UUID-Serial: " << nd_config.uuid_serial;
+    }
 
-    ostringstream site_uuid;
+    ostringstream uuid_site;
     if (strncmp(nd_config.uuid_site, ND_SITE_UUID_NULL, ND_SITE_UUID_LEN))
-        site_uuid << "X-UUID-Site: " << nd_config.uuid_site;
+        uuid_site << "X-UUID-Site: " << nd_config.uuid_site;
     else {
         string _uuid;
         if (nd_load_uuid(_uuid, nd_config.path_uuid_site, ND_SITE_UUID_LEN))
-            site_uuid << "X-UUID-Site: " << _uuid;
+            uuid_site << "X-UUID-Site: " << _uuid;
         else
-            site_uuid << "X-UUID-Site: " << nd_config.uuid_site;
+            uuid_site << "X-UUID-Site: " << nd_config.uuid_site;
     }
 
     string digest;
@@ -382,17 +386,25 @@ void ndSinkThread::CreateHeaders(void)
     headers = curl_slist_append(headers, user_agent.str().c_str());
     headers = curl_slist_append(headers, "Content-Type: application/json");
     headers = curl_slist_append(headers, uuid.str().c_str());
-    headers = curl_slist_append(headers, serial.str().c_str());
-    headers = curl_slist_append(headers, site_uuid.str().c_str());
+    headers = curl_slist_append(headers, uuid_serial.str().c_str());
+    headers = curl_slist_append(headers, uuid_site.str().c_str());
     headers = curl_slist_append(headers, conf_digest.str().c_str());
 
     headers_gz = curl_slist_append(headers_gz, user_agent.str().c_str());
     headers_gz = curl_slist_append(headers_gz, "Content-Type: application/json");
     headers_gz = curl_slist_append(headers_gz, "Content-Encoding: gzip");
     headers_gz = curl_slist_append(headers_gz, uuid.str().c_str());
-    headers_gz = curl_slist_append(headers_gz, serial.str().c_str());
-    headers_gz = curl_slist_append(headers_gz, site_uuid.str().c_str());
+    headers_gz = curl_slist_append(headers_gz, uuid_serial.str().c_str());
+    headers_gz = curl_slist_append(headers_gz, uuid_site.str().c_str());
     headers_gz = curl_slist_append(headers_gz, conf_digest.str().c_str());
+
+    for (map<string, string>::const_iterator i = nd_config.custom_headers.begin();
+        i != nd_config.custom_headers.end(); i++) {
+        ostringstream os;
+        os << (*i).first << ": " << (*i).second;
+        headers = curl_slist_append(headers, os.str().c_str());
+        headers_gz = curl_slist_append(headers_gz, os.str().c_str());
+    }
 }
 
 void ndSinkThread::FreeHeaders(void)
@@ -411,8 +423,20 @@ void ndSinkThread::FreeHeaders(void)
 void ndSinkThread::Upload(void)
 {
     CURLcode curl_rc;
+    bool flush_queue = true;
     size_t xfer = 0, total = pending.size();
 
+    if (post_errors == nd_config.sink_max_post_errors) {
+        free(nd_config.url_sink);
+        nd_config.url_sink = strdup(ND_URL_SINK);
+        nd_printf("%s: reverted to default sink URL: %s\n", tag.c_str(),
+            nd_config.url_sink);
+
+        curl_easy_setopt(ch, CURLOPT_URL, nd_config.url_sink);
+
+        post_errors = 0;
+    }
+
     do {
         nd_debug_printf("%s: payload %lu/%lu (%d of %d bytes)...\n",
             tag.c_str(), ++xfer, total, pending.front().second.size(), pending_size);
@@ -427,13 +451,39 @@ void ndSinkThread::Upload(void)
 
         body_data.clear();
 
-        if ((curl_rc = curl_easy_perform(ch)) != CURLE_OK)
+        if ((curl_rc = curl_easy_perform(ch)) != CURLE_OK) {
+
+            post_errors++;
+
+            ndJsonResponse *response = new ndJsonResponse(
+                ndJSON_RESP_POST_ERROR,
+                "Some POST error"
+            );
+
+            if (response == NULL)
+                throw runtime_error(strerror(ENOMEM));
+            else
+                PushResponse(response);
             break;
+        }
 
         long http_rc = 0;
         if ((curl_rc = curl_easy_getinfo(ch,
-            CURLINFO_RESPONSE_CODE, &http_rc)) != CURLE_OK)
+            CURLINFO_RESPONSE_CODE, &http_rc)) != CURLE_OK) {
+
+            post_errors++;
+
+            ndJsonResponse *response = new ndJsonResponse(
+                ndJSON_RESP_POST_ERROR,
+                "Some POST error"
+            );
+
+            if (response == NULL)
+                throw runtime_error(strerror(ENOMEM));
+            else
+                PushResponse(response);
             break;
+        }
 
         char *content_type = NULL;
         curl_easy_getinfo(ch, CURLINFO_CONTENT_TYPE, &content_type);
@@ -444,13 +494,37 @@ void ndSinkThread::Upload(void)
         if (content_type != NULL && content_length != 0.0f) {
             if (strcasecmp("application/json", content_type) == 0)
                 ProcessResponse();
+            else {
+                ndJsonResponse *response = new ndJsonResponse(
+                    ndJSON_RESP_INVALID_CONTENT_TYPE,
+                    "Invalid content type, expected: application/json"
+                );
+
+                if (response == NULL)
+                    throw runtime_error(strerror(ENOMEM));
+                else
+                    PushResponse(response);
+            }
+        }
+        else {
+            ndJsonResponse *response = new ndJsonResponse(
+                ndJSON_RESP_INVALID_RESPONSE,
+                "Unknown content type or invalid content length"
+            );
+
+            if (response == NULL)
+                throw runtime_error(strerror(ENOMEM));
+            else
+                PushResponse(response);
         }
 
         switch (http_rc) {
         case 200:
+            post_errors = 0;
             break;
 
         case 400:
+            post_errors = 0;
 #ifndef _ND_LEAN_AND_MEAN
             if (ND_DEBUG || ND_DEBUG_UPLOAD) {
                 FILE *hf = fopen(ND_JSON_FILE_BAD_SEND, "w");
@@ -464,16 +538,30 @@ void ndSinkThread::Upload(void)
                 }
             }
 #endif
-            break;
+        case 404:
+            post_errors++;
+            return;
 
         default:
+            post_errors = 0;
             return;
         }
 
         pending_size -= pending.front().second.size();
         pending.pop_front();
+
+        if (pending.size() == 0 || terminate)
+            flush_queue = false;
+        else {
+            Lock();
+
+            // Collect upload queue as soon as possible...
+            if (uploads.size() != 0) flush_queue = false;
+
+            Unlock();
+        }
     }
-    while (pending.size() > 0 && ! terminate);
+    while (flush_queue);
 }
 
 string ndSinkThread::Deflate(const string &data)
@@ -490,7 +578,8 @@ string ndSinkThread::Deflate(const string &data)
     if (deflateInit2(
         &zs,
         Z_DEFAULT_COMPRESSION,
-        Z_DEFLATED, 15 /* window bits */ | 16 /* enable GZIP format */,
+        Z_DEFLATED,
+        15 /* window bits */ | 16 /* enable GZIP format */,
         8,
         Z_DEFAULT_STRATEGY
     ) != Z_OK) throw ndSinkThreadException("deflateInit2");
@@ -530,8 +619,8 @@ void ndSinkThread::ProcessResponse(void)
 
         response->Parse(body_data);
 
-        switch (response->resp_code) {
-        case ndJSON_RESP_OK:
+        if (response->resp_code == ndJSON_RESP_OK) {
+
             if (response->uuid_site.size() == ND_SITE_UUID_LEN
                 && nd_save_uuid(
                     response->uuid_site,
@@ -543,6 +632,18 @@ void ndSinkThread::ProcessResponse(void)
                 create_headers = true;
             }
 
+            if (response->url_sink.size() &&
+                response->url_sink != nd_config.url_sink) {
+                if (nd_save_sink_url(response->url_sink)) {
+                    free(nd_config.url_sink);
+                    nd_config.url_sink = strdup(response->url_sink.c_str());
+                    nd_printf("%s: saved new sink URL: %s\n", tag.c_str(),
+                        response->url_sink.c_str());
+
+                    curl_easy_setopt(ch, CURLOPT_URL, nd_config.url_sink);
+                }
+            }
+
             for (ndJsonData::const_iterator i = response->data.begin();
                 i != response->data.end(); i++) {
 
@@ -553,34 +654,26 @@ void ndSinkThread::ProcessResponse(void)
                             nd_config.path_sink_config, nd_config.digest_sink_config
                         ) == 0)
                         create_headers = true;
-
-                    break;
                 }
             }
 
             if (create_headers) CreateHeaders();
-
-            PushResponse(response);
-
-            nd_debug_printf("%s: [%d] %s\n", tag.c_str(),
-                response->resp_code,
-                (response->resp_message.size() > 0) ?
-                    response->resp_message.c_str() : "(no message)");
-            break;
-
-        default:
-            nd_printf("%s: [%d] %s\n", tag.c_str(),
-                response->resp_code,
-                (response->resp_message.size() > 0) ?
-                    response->resp_message.c_str() : "(no message)");
-            if (response != NULL) delete response;
         }
     } catch (ndJsonParseException &e) {
-        if (response != NULL) delete response;
-        nd_printf("JSON response parse error: %s\n", e.what());
+        response->resp_code = ndJSON_RESP_PARSE_ERROR;
+        response->resp_message = e.what();
     } catch (runtime_error &e) {
         nd_printf("JSON response parse error: %s\n", e.what());
     }
+
+    if (response != NULL) {
+        nd_debug_printf("%s: [%d] %s\n", tag.c_str(),
+            response->resp_code,
+            (response->resp_message.size() > 0) ?
+                response->resp_message.c_str() : "(no message)");
+
+        PushResponse(response);
+    }
 }
 
 // vi: expandtab shiftwidth=4 softtabstop=4 tabstop=4
