@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2018 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2019 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include <iostream>
 #include <map>
 #include <queue>
+#include <list>
 #include <sstream>
 #include <stdexcept>
 #include <string>
@@ -31,9 +32,8 @@
 #include <vector>
 #ifdef HAVE_ATOMIC
 #include <atomic>
-#else
-typedef bool atomic_bool;
 #endif
+#include <regex>
 
 #include <arpa/inet.h>
 #include <errno.h>
@@ -58,12 +58,10 @@ typedef bool atomic_bool;
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/un.h>
-#ifdef _ND_USE_NETLINK
-#include <linux/netlink.h>
-#endif
+#include <sys/param.h>
 
 #ifndef UNIX_PATH_MAX
-#define UNIX_PATH_MAX 108
+#define UNIX_PATH_MAX 104
 #endif
 
 #include <pcap/pcap.h>
@@ -88,6 +86,7 @@ using namespace std;
 #include "nd-conntrack.h"
 #endif
 #include "nd-util.h"
+#include "nd-signal.h"
 #include "nd-socket.h"
 
 #define _ND_SOCKET_PROC_NET_UNIX    "/proc/net/unix"
@@ -126,6 +125,19 @@ ndSocketLocal::~ndSocketLocal()
         unlink(base->node.c_str());
 }
 
+#ifdef BSD4_4
+int ndSocketLocal::IsValid(void)
+{
+    // TODO: Need a "BSD-way" to achieve the same...
+    int rc = unlink(base->node.c_str());
+    if (rc != 0 && errno != ENOENT) {
+        nd_printf("Error while removing stale socket file: %s: %s\n",
+            base->node.c_str(), strerror(errno));
+        return rc;
+    }
+    return 0;
+}
+#else
 int ndSocketLocal::IsValid(void)
 {
     //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
@@ -175,6 +187,7 @@ int ndSocketLocal::IsValid(void)
 
     return 0;
 }
+#endif
 
 ndSocketRemote::ndSocketRemote(
     ndSocket *base, const string &node, const string &service)
@@ -510,7 +523,7 @@ void ndSocket::Create(void)
                         "setsockopt: SO_REUSEADDR", errno);
                 }
 
-                if (bind(sd, rp->ai_addr, rp->ai_addrlen) == 0) break;
+                if (::bind(sd, rp->ai_addr, rp->ai_addrlen) == 0) break;
                 else {
                     nd_printf("%s: bind: %s\n",
                         __PRETTY_FUNCTION__, strerror(errno));
@@ -551,7 +564,7 @@ void ndSocket::Create(void)
             nd_printf("%s: connected\n", __PRETTY_FUNCTION__);
         }
         else if (type == ndSOCKET_TYPE_SERVER) {
-            if (bind(sd, sa, sa_size) != 0)
+            if (::bind(sd, sa, sa_size) != 0)
                 throw ndSocketSystemException(__PRETTY_FUNCTION__, "bind", errno);
 
             if (listen(sd, SOMAXCONN) != 0)
@@ -562,46 +575,100 @@ void ndSocket::Create(void)
     nd_debug_printf("%s: created\n", __PRETTY_FUNCTION__);
 }
 
-size_t ndSocketBuffer::GetLength(void)
+ndSocketBuffer::ndSocketBuffer()
+    : buffer(NULL), fd_fifo{-1, -1},
+    buffer_queue_offset(0), buffer_queue_length(0)
 {
-    if (buffer.size() == 0) return 0;
-    return length - offset;
+    buffer = new uint8_t[_ND_SOCKET_BUFSIZE];
+
+    if (buffer == NULL)
+        throw ndSocketSystemException(__PRETTY_FUNCTION__, "new", errno);
+    if (socketpair(AF_LOCAL, SOCK_STREAM | SOCK_NONBLOCK, 0, fd_fifo) < 0)
+        throw ndSocketSystemException(__PRETTY_FUNCTION__, "socketpair", errno);
 }
 
-const uint8_t *ndSocketBuffer::GetBuffer(ssize_t &length)
+ndSocketBuffer::~ndSocketBuffer()
 {
-    if (buffer.size() == 0) {
-        length = 0;
-        return NULL;
+    if (buffer != NULL) delete [] buffer;
+    if (fd_fifo[0] != -1) close(fd_fifo[0]);
+    if (fd_fifo[1] != -1) close(fd_fifo[1]);
+}
+
+size_t ndSocketBuffer::BufferQueueFlush(void)
+{
+    ssize_t bytes_wrote = 0;
+    size_t bytes = 0, bytes_flushed = 0;
+
+    while (buffer_queue.size()) {
+
+        bytes = buffer_queue.front().size() - buffer_queue_offset;
+        bytes_wrote = send(fd_fifo[1], buffer_queue.front().c_str() + buffer_queue_offset, bytes, 0);
+
+        if (bytes_wrote < 0) {
+            if (errno == EAGAIN || errno == EWOULDBLOCK) {
+                //nd_debug_printf("Unable to flush buffer queue to client socket: %s\n",
+                //    strerror(errno));
+                break;
+            }
+            throw ndSocketSystemException(__PRETTY_FUNCTION__, "send", errno);
+        }
+
+        bytes_flushed += bytes_wrote;
+
+        if ((size_t)bytes_wrote != bytes)
+            buffer_queue_offset += bytes_wrote;
+        else {
+            buffer_queue_offset = 0;
+            buffer_queue.pop_front();
+        }
     }
 
-    length = buffer.front().size() - offset;
-    return (const uint8_t *)(buffer.front().c_str() + offset);
+    buffer_queue_length -= bytes_flushed;
+
+    return bytes_flushed;
+}
+
+const uint8_t *ndSocketBuffer::GetBuffer(ssize_t &bytes)
+{
+    bytes = recv(fd_fifo[0], buffer, _ND_SOCKET_BUFSIZE, MSG_PEEK);
+    if (bytes < 0) {
+        if (errno == EAGAIN || errno == EWOULDBLOCK) {
+            bytes = 0;
+            return NULL;
+        }
+        throw ndSocketSystemException(__PRETTY_FUNCTION__, "recv", errno);
+    }
+    else if (bytes == 0)
+        throw ndSocketHangupException("recv");
+
+    return (const uint8_t *)buffer;
 }
 
 void ndSocketBuffer::Push(const string &data)
 {
-    ostringstream header;
-    header << "{\"length\": " << data.size() << "}\n";
-    buffer.push_back(header.str());
-    buffer.push_back(data);
-    length += data.size();
+    ostringstream payload;
+    payload << "{\"length\": " << data.size() << "}\n";
+    payload << data;
+
+    buffer_queue.push_back(payload.str());
+    buffer_queue_length += payload.str().size();
+
+    BufferQueueFlush();
 }
 
-void ndSocketBuffer::Pop(ssize_t length)
+void ndSocketBuffer::Pop(size_t bytes)
 {
-    if (length <= 0 || buffer.size() == 0) return;
-    size_t bytes = buffer.front().size() - offset;
-    if (bytes <= 0) return;
-    if ((size_t)length > bytes) return;
-    if ((size_t)length == bytes) {
-        offset = 0;
-        buffer.pop_front();
-    }
-    else if ((size_t)length < bytes)
-        offset += length;
+    if (bytes == 0 || bytes > _ND_SOCKET_BUFSIZE)
+        throw ndSocketSystemException(__PRETTY_FUNCTION__, "invalid size", EINVAL);
 
-    length -= bytes;
+    ssize_t bytes_recv = recv(fd_fifo[0], buffer, bytes, 0);
+
+    if (bytes_recv < 0)
+        throw ndSocketSystemException(__PRETTY_FUNCTION__, "recv", errno);
+    else if (bytes_recv == 0)
+        throw ndSocketHangupException("recv");
+    else if ((size_t)bytes_recv != bytes)
+        throw ndSocketSystemException(__PRETTY_FUNCTION__, "recv(short)", EINVAL);
 }
 
 ndSocketThread::ndSocketThread()
@@ -629,7 +696,7 @@ ndSocketThread::~ndSocketThread()
 {
     Join();
 
-    for (ndSocketMap::const_iterator i = clients.begin();
+    for (ndSocketClientMap::const_iterator i = clients.begin();
         i != clients.end(); i++) {
         delete i->second;
     }
@@ -645,8 +712,13 @@ ndSocketThread::~ndSocketThread()
 
 void ndSocketThread::QueueWrite(const string &data)
 {
+    ndSocketBufferMap::iterator bi;
+
     Lock();
-    queue_write.push_back(data);
+
+    for (bi = buffers.begin(); bi != buffers.end(); bi++)
+        bi->second->Push(data);
+
     Unlock();
 }
 
@@ -670,6 +742,8 @@ void ndSocketThread::ClientAccept(ndSocketServerMap::iterator &si)
         throw ndSocketThreadException(__PRETTY_FUNCTION__, "new", ENOMEM);
     }
 
+    Lock();
+
     buffers[client->GetDescriptor()] = buffer;
     clients[client->GetDescriptor()] = client;
 
@@ -680,13 +754,17 @@ void ndSocketThread::ClientAccept(ndSocketServerMap::iterator &si)
     buffer->Push(json);
     nd_json_protocols(json);
     buffer->Push(json);
+
+    Unlock();
+
+    kill(getpid(), ND_SIG_CONNECT);
 }
 
-void ndSocketThread::ClientHangup(ndSocketMap::iterator &ci)
+void ndSocketThread::ClientHangup(ndSocketClientMap::iterator &ci)
 {
     ndSocketBufferMap::iterator bi;
 
-    nd_printf("%s\n", __PRETTY_FUNCTION__);
+    nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
 
     delete ci->second;
     bi = buffers.find(ci->first);
@@ -697,43 +775,38 @@ void ndSocketThread::ClientHangup(ndSocketMap::iterator &ci)
             __PRETTY_FUNCTION__, "buffers.find", ENOENT);
     }
     else {
+        Lock();
+
         delete bi->second;
         buffers.erase(bi);
+
+        Unlock();
     }
 }
 
 void *ndSocketThread::Entry(void)
 {
-    int rc, max_fd;
-    fd_set fds_read, fds_write;
     struct timeval tv;
-    ndSocketMap::iterator ci;
+    fd_set fds_read, fds_write;
+    ndSocketClientMap::iterator ci;
     ndSocketServerMap::iterator si;
     ndSocketBufferMap::iterator bi;
 
     nd_debug_printf("%s: started\n", __PRETTY_FUNCTION__);
 
     while (! terminate) {
-        max_fd = -1;
+        int rc_read = -1, rc_write = -1;
+        int max_read_fd = -1, max_write_fd = -1;
 
         FD_ZERO(&fds_read);
         FD_ZERO(&fds_write);
 
-        Lock();
-        if (queue_write.size()) {
-            for (vector<string>::iterator i = queue_write.begin();
-                i != queue_write.end(); i++) {
-                for (bi = buffers.begin(); bi != buffers.end(); bi++)
-                    bi->second->Push((*i));
-            }
-            queue_write.clear();
-        }
-        Unlock();
-
         for (ci = clients.begin(); ci != clients.end(); ci++) {
 
             FD_SET(ci->first, &fds_read);
-            if (ci->first > max_fd) max_fd = ci->first;
+            FD_SET(ci->first, &fds_write);
+            if (ci->first > max_read_fd) max_read_fd = ci->first;
+            if (ci->first > max_write_fd) max_write_fd = ci->first;
 
             bi = buffers.find(ci->first);
             if (bi == buffers.end()) {
@@ -741,66 +814,103 @@ void *ndSocketThread::Entry(void)
                     __PRETTY_FUNCTION__, "buffers.find", ENOENT);
             }
 
-            if (bi->second->GetLength() > 0)
-                FD_SET(ci->first, &fds_write);
+            int fd = bi->second->GetDescriptor();
+            FD_SET(fd, &fds_read);
+            if (fd > max_read_fd) max_read_fd = fd;
         }
 
         for (si = servers.begin(); si != servers.end(); si++) {
-
             FD_SET(si->first, &fds_read);
-            if (si->first > max_fd) max_fd = si->first;
+            if (si->first > max_read_fd) max_read_fd = si->first;
         }
 
         memset(&tv, 0, sizeof(struct timeval));
         tv.tv_sec = 1;
 
-        rc = select(max_fd + 1, &fds_read, &fds_write, NULL, &tv);
+        rc_read = select(max_read_fd + 1, &fds_read, NULL, NULL, &tv);
 
-        if (rc == -1 && errno != EINTR) {
+        if (rc_read == -1 && errno != EINTR) {
             throw ndSocketThreadException(
-                __PRETTY_FUNCTION__, "select", errno);
+                __PRETTY_FUNCTION__, "select for read", errno);
         }
 
-        if (rc == 0) continue;
+        if (rc_read == 0) continue;
+
+        if (clients.size()) {
+            memset(&tv, 0, sizeof(struct timeval));
+
+            rc_write = select(max_write_fd + 1, NULL, &fds_write, NULL, &tv);
+
+            if (rc_write == -1 && errno != EINTR) {
+                throw ndSocketThreadException(
+                    __PRETTY_FUNCTION__, "select for write", errno);
+            }
+        }
 
         ci = clients.begin();
 
-        while (ci != clients.end()) {
+        while (rc_write > 0 && ci != clients.end()) {
+
+            bool hangup = false;
 
             if (FD_ISSET(ci->first, &fds_read)) {
                 ClientHangup(ci);
-                if (--rc == 0) break;
+                if (--rc_read == 0) break;
                 continue;
             }
 
-            if (FD_ISSET(ci->first, &fds_write)) {
-
-                bi = buffers.find(ci->first);
-                if (bi == buffers.end()) {
-                    throw ndSocketThreadException(__PRETTY_FUNCTION__,
-                        "buffers.find", ENOENT);
-                }
-
-                ssize_t length;
-                const uint8_t *p = bi->second->GetBuffer(length);
-
-                try {
-                    ssize_t bytes = ci->second->Write(p, length);
-                    bi->second->Pop(bytes);
-                } catch (ndSocketHangupException &e) {
-                    ClientHangup(ci);
-                } catch (ndSocketSystemException &e) {
-                    ClientHangup(ci);
-                }
-
-                if (--rc == 0) break;
-                continue;
+            bi = buffers.find(ci->first);
+            if (bi == buffers.end()) {
+                throw ndSocketThreadException(__PRETTY_FUNCTION__,
+                    "buffers.find", ENOENT);
             }
 
-            ci++;
+            if (FD_ISSET(bi->second->GetDescriptor(), &fds_read) &&
+                FD_ISSET(ci->first, &fds_write)) {
+
+                rc_write--;
+
+                ssize_t length = 0;
+                const uint8_t *p = NULL;
+
+                Lock();
+                bi->second->BufferQueueFlush();
+                Unlock();
+
+                p = bi->second->GetBuffer(length);
+
+                while (p != NULL && length > 0) {
+                    try {
+                        ssize_t bytes = ci->second->Write(p, length);
+
+                        bi->second->Pop(bytes);
+
+                        if (bytes != length) break;
+
+                    } catch (ndSocketHangupException &e) {
+                        hangup = true;
+                        ClientHangup(ci);
+                        break;
+                    } catch (ndSocketSystemException &e) {
+                        hangup = true;
+                        ClientHangup(ci);
+                        break;
+                    }
+
+                    Lock();
+                    bi->second->BufferQueueFlush();
+                    Unlock();
+
+                    p = bi->second->GetBuffer(length);
+                }
+
+                if (--rc_read == 0) break;
+            }
+
+            if (! hangup) ci++;
         }
 
-        if (rc == 0) continue;
+        if (rc_read == 0) continue;
 
         for (si = servers.begin(); si != servers.end(); si++) {
             if (FD_ISSET(si->first, &fds_read)) {
@@ -811,7 +921,7 @@ void *ndSocketThread::Entry(void)
                         tag.c_str(), e.what());
                 }
 
-                if (--rc == 0) break;
+                if (--rc_read == 0) break;
             }
         }
     }
