@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2018 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2019 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,20 +24,22 @@
 #include <unordered_map>
 #ifdef HAVE_ATOMIC
 #include <atomic>
-#else
-typedef bool atomic_bool;
 #endif
+#include <regex>
 
 #include <unistd.h>
 #include <stdlib.h>
 #include <signal.h>
 #include <pthread.h>
+#include <string.h>
+#include <errno.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
 
 #include <json.h>
+#include <pcap/pcap.h>
 
 using namespace std;
 
@@ -75,8 +77,9 @@ static void *nd_thread_entry(void *param)
     return rv;
 }
 
-ndThread::ndThread(const string &tag, long cpu)
-    : tag(tag), id(0), cpu(cpu), terminate(false), terminated(false)
+ndThread::ndThread(const string &tag, long cpu, bool ipc)
+    : tag(tag), id(0), cpu(cpu), terminate(false), terminated(false),
+    fd_ipc{-1, -1}
 {
     int rc;
 
@@ -86,6 +89,9 @@ ndThread::ndThread(const string &tag, long cpu)
     if ((rc = pthread_mutex_init(&lock, NULL)) != 0)
         throw ndThreadException(strerror(rc));
 
+    if (ipc && socketpair(AF_LOCAL, SOCK_STREAM | SOCK_NONBLOCK, 0, fd_ipc) < 0)
+        throw ndThreadSystemException(__PRETTY_FUNCTION__, "socketpair", errno);
+
     if (cpu == -1) return;
 #if defined(HAVE_PTHREAD_ATTR_SETAFFINITY_NP) && defined(CPU_ALLOC)
     long cpus = sysconf(_SC_NPROCESSORS_ONLN);
@@ -114,6 +120,9 @@ ndThread::~ndThread(void)
 {
     pthread_attr_destroy(&attr);
     pthread_mutex_destroy(&lock);
+
+    if (fd_ipc[0] != -1) close(fd_ipc[0]);
+    if (fd_ipc[1] != -1) close(fd_ipc[1]);
 }
 
 void ndThread::SetProcName(void)
@@ -171,4 +180,31 @@ void ndThread::Unlock(void)
         throw ndThreadException(strerror(rc));
 }
 
+void ndThread::SendIPC(uint32_t id)
+{
+    ssize_t bytes_wrote = 0;
+
+    bytes_wrote = send(fd_ipc[1], &id, sizeof(uint32_t), 0);
+
+    if (bytes_wrote != sizeof(uint32_t)) {
+        nd_debug_printf("%s: Failed to send IPC message: %s\n",
+            tag.c_str(), strerror(errno));
+    }
+}
+
+uint32_t ndThread::RecvIPC(void)
+{
+    uint32_t id = 0;
+    ssize_t bytes_read = 0;
+
+    bytes_read = recv(fd_ipc[0], &id, sizeof(uint32_t), 0);
+
+    if (bytes_read != sizeof(uint32_t)) {
+        nd_debug_printf("%s: Failed to receive IPC message: %s\n",
+            tag.c_str(), strerror(errno));
+    }
+
+    return id;
+}
+
 // vi: expandtab shiftwidth=4 softtabstop=4 tabstop=4
